#include "CGame.h"

void CGame::Initialize()
{
    SetupGPUProgram();
    SetupVAO();
}

void CGame::Shutdown()
{
    
}

void CGame::OnResize(int width, int height)
{
    glViewport(0, 0, width, height);
}

void CGame::Render()
{
    glClearColor(0.2f, 0.3f, 0.3f, 1.0f);
    glClear(GL_COLOR_BUFFER_BIT);

    m_pShader->use();
    glBindVertexArray(m_VBO);
    glDrawArrays(GL_TRIANGLES, 0, 3);
}

void CGame::SetupVAO()
{
    float vertices[] = {
        -0.5f, -0.5f, 0.0f,     1.0f, 0.0f, 0.0f,
        0.5f, -0.5f, 0.0f,      0.0f, 1.0f, 0.0f,
        0.0f,  0.5f, 0.0f,      0.0f, 0.0f, 1.0f
    };

    glGenBuffers(1, &m_VBO);

    glGenVertexArrays(1, &m_VAO);
    // 1. bind Vertex Array Object
    glBindVertexArray(m_VAO);
    // 2. copy our vertices array in a buffer for OpenGL to use
    glBindBuffer(GL_ARRAY_BUFFER, m_VBO);

    // Allocate enough memory to hold our vertex buffer.
    // Don't forget the GL_MAP_WRITE_BIT!
    glBufferStorage(GL_ARRAY_BUFFER, sizeof(vertices), NULL, GL_MAP_WRITE_BIT);

    // Map the vertex data from the CPU to the GPU buffer.
    int count = sizeof(vertices);
    void* data = glMapBuffer(GL_ARRAY_BUFFER, GL_WRITE_ONLY);
    memcpy(data, vertices, count);
    glUnmapBuffer(GL_ARRAY_BUFFER);

    // 3. then set our vertex attributes pointers
    // Position Attribute.  The offset is 0 since the position is at the start of thr array.
    glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 6 * sizeof(float), (void*)0);
    glEnableVertexAttribArray(0);

    // Color Attribute.  The offset is 3 * sizeof(float) because the first color comes after the
    // first vertex position which is of size 3 * sizeof(float).
    glVertexAttribPointer(1, 3, GL_FLOAT, GL_FALSE, 6 * sizeof(float), (void*)(3 * sizeof(float)));
    glEnableVertexAttribArray(1);
}

void CGame::SetupGPUProgram()
{
    m_pShader = new Shader("vertex.glsl", "fragment.glsl");
}